@@ -113,7 +113,7 @@ private:
     if(last_message_time){
       std::chrono::duration<double> tmp = *now - *last_message_time;
       elapsed_seconds = tmp.count();
-    } else{
+    } else {
       elapsed_seconds = std::numeric_limits<double>::quiet_NaN();
 
     }
@@ -128,50 +128,179 @@ private:
   }
 };
 
-/* data class for pre-computed factorization */
+/* data class for pre-computed factorization and matrices which are only
+ * computed once */
 class covarmat {
+private:
+#ifdef _OPENMP
+  std::unique_ptr<omp_lock_t> lock =
+    std::unique_ptr<omp_lock_t>((new omp_lock_t()));
+  class LockGuard {
+  public:
+    explicit LockGuard(omp_lock_t& lock) : m_lock(lock){
+      omp_set_lock(&m_lock);
+    }
+
+    ~LockGuard() {
+      omp_unset_lock(&m_lock);
+    }
+
+  private:
+    omp_lock_t& m_lock;
+  };
+#endif
+
+  enum output { e_mat, e_chol, e_chol_inv, e_inv };
+
+  std::unique_ptr<const arma::mat> mat_;
+  std::unique_ptr<bool> is_chol_set =
+    std::unique_ptr<bool>(new bool(false));
+  std::unique_ptr<arma::mat> chol_;
+  std::unique_ptr<bool> is_chol_inv_set =
+    std::unique_ptr<bool>(new bool(false));
+  std::unique_ptr<arma::mat> chol_inv_;
+  std::unique_ptr<bool> is_inv_set =
+    std::unique_ptr<bool>(new bool(false));
+  std::unique_ptr<arma::mat> inv_;
+
+  const arma::mat& get_mat(output what) const {
+    if(what == e_mat)
+      return(*mat_.get());
+
+    bool is_computed, *this_flag;
+
+    this_flag = is_chol_set.get();
+#ifdef _OPENMP
+#pragma omp atomic read
+    is_computed = *this_flag;
+    if(!is_computed){
+      LockGuard guard(*lock.get());
+#pragma omp atomic read
+      is_computed = *this_flag;
+      if(!is_computed){
+        *chol_.get() += arma::chol(*mat_.get());
+#pragma omp atomic write
+        *this_flag = true;
+      }
+    }
+#else
+    if(!*this_flag){
+      *chol_.get() += arma::chol(*mat_.get());
+      *this_flag = true;
+    }
+#endif
+    if(what == e_chol)
+      return(*chol_.get());
+
+
+    this_flag = is_chol_inv_set.get();
+#ifdef _OPENMP
+#pragma omp atomic read
+    is_computed = *this_flag;
+    if(!is_computed){
+      LockGuard guard(*lock.get());
+#pragma omp atomic read
+      is_computed = *this_flag;
+      if(!is_computed){
+        *chol_inv_.get() += arma::inv(arma::trimatu(*chol_.get()));
+#pragma omp atomic write
+        *this_flag = true;
+      }
+    }
+#else
+    if(!*this_flag){
+      *chol_inv_.get() += arma::inv(arma::trimatu(*chol_.get()));
+      *this_flag = true;
+    }
+#endif
+    if(what == e_chol_inv)
+      return(*chol_inv_.get());
+
+
+    this_flag = is_inv_set.get();
+#ifdef _OPENMP
+#pragma omp atomic read
+    is_computed = *this_flag;
+    if(!is_computed){
+      LockGuard guard(*lock.get());
+#pragma omp atomic read
+      is_computed = *this_flag;
+      if(!is_computed){
+        *inv_.get() += mat_->i();
+#pragma omp atomic write
+        *this_flag = true;
+      }
+    }
+#else
+    if(!*this_flag){
+      *inv_.get() += mat_->i();
+      *this_flag = true;
+    }
+#endif
+    return(*inv_.get());
+  }
+
 public:
   /* Covar matrix: Q */
-  const arma::mat mat;
+  const arma::mat& mat() const  {
+    return get_mat(e_mat);
+  }
   /* C in Q = C^\topC in  */
-  const arma::mat chol;
+  const arma::mat& chol() const {
+    return get_mat(e_chol);
+  }
   /* C^{-1} */
-  const arma::mat chol_inv;
+  const arma::mat& chol_inv() const {
+    return get_mat(e_chol_inv);
+  }
   /* Q^{-1} */
-  const arma::mat inv;
+  const arma::mat& inv() const {
+    return get_mat(e_inv);
+  }
 
   template<typename T>
   covarmat(T Q):
-    mat(Q), chol(arma::chol(mat)), chol_inv(arma::inv(arma::trimatu(chol))),
-    inv(arma::inv(Q))
-  {}
+    mat_(new arma::mat(Q)),
+    chol_    (new arma::mat(arma::size(Q), arma::fill::zeros)),
+    chol_inv_(new arma::mat(arma::size(Q), arma::fill::zeros)),
+    inv_     (new arma::mat(arma::size(Q), arma::fill::zeros)) {
+#ifdef _OPENMP
+    omp_init_lock(lock.get());
+#endif
+  }
+
+  covarmat(const covarmat &other): covarmat(other.mat()) { }
+
+  ~covarmat(){
+#ifdef _OPENMP
+    omp_destroy_lock(lock.get());
+#endif
+  }
 };
 
 // data holder for particle filtering
 class PF_data : public problem_data {
   using uword = arma::uword;
 
-  /* Pre-computed matrices and mapping function for Taylor approximation used
-   * in the [b]ack[w]ard filter */
+  /* Objects used to compute P(\alpha_t \vert \alpha_{t - 1}) */
   std::map<const uword, const std::unique_ptr<linear_mapper>> bw_mean_maps;
   std::map<const uword, const arma::vec> bw_mean_const_term;
   std::map<const uword, const covarmat> bw_covar_map;
 
-  /* Unconditional mean variance for the entire state */
-  std::map<const uword, const arma::vec> uncond_mean_state_map;
-  std::map<const uword, const covarmat>  uncond_covar_state_map;
+  /* Pre-computed objects for the artificial prior */
+  std::map<const uword, const arma::vec> uncond_means;
+  std::map<const uword, const covarmat>  uncond_covarmats;
 
-  /* uncondtional mean and covariance matrix for the terms that are affected
-   * by the inovations. E.g.,
-   *   R^\top\alpha */
-  std::map<const uword, const arma::vec> uncond_mean_map;
-  std::map<const uword, const arma::mat> uncond_covar_map;
+  /* uncondtional mean and covariance matrix terms in backward filter's
+   * and smoother's proposal distribution in the Taylor approximation */
+  std::map<const uword, const arma::vec> uncond_mean_terms;
+  std::map<const uword, const arma::mat> uncond_covs_inv;
 
 protected:
   virtual void set_maps(){
     arma::vec m_t, m_t_p_1 = a_0;
-    arma::mat Q_state = err_state->map(Q.mat).sv;
-    arma::mat P_t, P_t_p_1 = Q_0.mat;
+    arma::mat Q_state = err_state->map(Q.mat()).sv;
+    arma::mat P_t, P_t_p_1 = Q_0.mat();
     for(int t = 0; t <= d + 1; ++t){
 
       // move and update
@@ -181,14 +310,16 @@ protected:
       P_t_p_1 = state_trans->map(P_t).sv + Q_state;
 
       // insert map elements
-      uncond_mean_state_map.insert(std::make_pair(t + 1L, m_t_p_1));
-      uncond_covar_state_map.insert(std::make_pair(t + 1L, covarmat(P_t_p_1)));
+      uncond_means.insert(std::make_pair(t + 1L, m_t_p_1));
+      uncond_covarmats.insert(std::make_pair(t + 1L, covarmat(P_t_p_1)));
 
       arma::mat S_t =
-        arma::solve(P_t_p_1, err_state->map(Q.mat).sv);
+        arma::solve(P_t_p_1, Q_state);
       S_t = state_trans_inv->map(S_t, right).sv;
       S_t = state_trans->map(P_t, right).sv * S_t;
 
+      /* 1. P_{t +1}^{-\top}F
+       * 2. P_t(P_{t +1}^{-\top}F)^\top */
       arma::mat tmp = arma::solve(P_t_p_1.t(), state_trans->map());
       bw_mean_maps.insert(std::make_pair(
           t, std::unique_ptr<dens_mapper>(
@@ -199,11 +330,18 @@ protected:
 
       bw_covar_map.insert(std::make_pair(t, std::move(S_t)));
 
-      uncond_mean_map.insert(std::make_pair(
-          t, err_state_inv->map(arma::vec(arma::solve(P_t, m_t))).sv));
+      uncond_mean_terms.insert(std::make_pair(
+          t, arma::vec(arma::solve(P_t, m_t))));
 
-      uncond_covar_map.insert(std::make_pair(
-          t, err_state_inv->map(arma::mat(P_t.i())).sv));
+      uncond_covs_inv.insert(std::make_pair(
+          t, arma::mat(P_t.i())));
+
+      if(debug > 4)
+        log(5) << "P_" << t << std::endl << P_t
+               << "m_" << t << std::endl << m_t.t()
+               << "S_" << t << std::endl << bw_covar_map.at(t).mat()
+               << "P(a_" << t << " | a_" << t  + 1 << ") mean term"
+               << std::endl << uncond_mean_terms.at(t).t();
     }
   }
 
@@ -223,7 +361,7 @@ public:
   const covarmat Q;
   const covarmat Q_0;
   const covarmat Q_proposal;
-  const covarmat Q_proposal_smooth;
+  const covarmat Q_proposal_state;
 
   PF_data(const int n_fixed_terms_in_state_vec,
           arma::mat &X,
@@ -233,7 +371,6 @@ public:
           const arma::colvec &a_0,
           const arma::mat &R,
           const arma::mat &L,
-          const arma::vec &m,
           arma::mat &Q_0,
           arma::mat &Q,
           const Rcpp::List &risk_obj,
@@ -251,7 +388,7 @@ public:
           const uword N_first) :
     problem_data(
       n_fixed_terms_in_state_vec,
-      X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, L, m, Q_0, Q,
+      X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, L, Q_0, Q,
       risk_obj, F, n_max, n_threads, fixed_parems),
 
       N_fw_n_bw(N_fw_n_bw),
@@ -268,13 +405,13 @@ public:
 
       Q(Q),
       Q_0(Q_0),
-      Q_proposal(Q + Q_tilde),
-      Q_proposal_smooth((Q + Q_tilde) * .5)
+      Q_proposal(Q_tilde),
+      Q_proposal_state(err_state->map(Q_tilde).sv)
     {
 #ifdef _OPENMP
-    omp_init_lock(&PF_logger::lock);
+      omp_init_lock(&PF_logger::lock);
 #endif
-    set_maps();
+      set_maps();
     }
 
   ~PF_data(){
@@ -300,19 +437,19 @@ public:
   }
 
   const arma::vec& uncond_mean_state(const uword t) const {
-    return uncond_mean_state_map.at(t);
+    return uncond_means.at(t);
   }
 
   const covarmat& uncond_covar_state(const uword t) const {
-    return uncond_covar_state_map.at(t);
+    return uncond_covarmats.at(t);
   }
 
-  const arma::vec& uncond_mean(const int t) const {
-    return uncond_mean_map.at(t);
+  const arma::vec& uncond_mean_term(const int t) const {
+    return uncond_mean_terms.at(t);
   }
 
-  const arma::mat& uncond_covar(const uword t) const {
-    return uncond_covar_map.at(t);
+  const arma::mat& uncond_covar_inv(const uword t) const {
+    return uncond_covs_inv.at(t);
   }
 };
 
