@@ -41,19 +41,18 @@ protected:
 */
 
 template<
-    template <typename, bool> class T_resampler,
-    template <typename, bool> class T_importance_dens,
-    class densities,
+    template <bool> class T_resampler,
+    template <bool> class T_importance_dens,
     bool is_forward>
 class AUX_PF : private PF_base {
-  using resampler = T_resampler<densities, is_forward>;
-  using importance_dens = T_importance_dens<densities, is_forward>;
+  using resampler = T_resampler<is_forward>;
+  using importance_dens = T_importance_dens<is_forward>;
 
 public:
-  static std::vector<cloud> compute(const PF_data &data){
+  static std::vector<cloud>
+  compute(const PF_data &data, pf_base_dens &dens_calc){
     std::vector<cloud> clouds;
     std::string direction_str = (is_forward) ? "forward" : "backward";
-    densities dens_calc = densities(data);
 
     if(data.debug > 0)
       data.log(1) << "Running " << direction_str << " filter"
@@ -93,30 +92,25 @@ public:
       if(data.debug > 0)
         data.log(1) << "Updating weights";
       {
+          const bool do_debug = data.debug > 4;
           double max_weight =  -std::numeric_limits<double>::max();
           arma::uvec r_set = get_risk_set(data, t);
           unsigned int n_elem = new_cloud.size();
 
 #ifdef _OPENMP
-#pragma omp parallel
-{
-#endif
-          double my_max_weight = -std::numeric_limits<double>::max();
-
-#ifdef _OPENMP
-#pragma omp for schedule(static)
+#pragma omp parallel for schedule(static) reduction(max:max_weight)
 #endif
           for(unsigned int i = 0; i < n_elem; ++i){ // loop over new particles
             auto it = new_cloud.begin() + i;
             double log_prob_y_given_state =
               dens_calc.log_prob_y_given_state(
-                data, it->get_state(), t, r_set, false);
+                it->get_state(), t, r_set, false);
             double log_prob_state_given_other =
               is_forward ?
               dens_calc.log_prob_state_given_previous(
-                data, it->get_state(), it->parent->get_state(), t) :
-              dens_calc.log_prob_state_given_next(
-                data, it->get_state(), it->parent->get_state(), t);
+                it->get_state()        , it->parent->get_state(), t) :
+              dens_calc.log_prob_state_given_previous(
+                it->parent->get_state(), it->get_state()        , t + 1);
 
             it->log_unnormalized_weight = it->log_weight =
               /* nominator */
@@ -140,20 +134,31 @@ public:
                 dens_calc.log_artificial_prior(*it->parent, t + 1);
             }
 
-            my_max_weight = MAX(it->log_weight, my_max_weight);
+            max_weight = MAX(max_weight, it->log_weight);
+
+            if(do_debug){
+              const int wd = 11;
+              std::stringstream ss;
+
+              ss << std::setprecision(6)
+                 << "log-like terms"
+                 << " 'log_prob_y_given_state' "           << std::setw(wd) << log_prob_y_given_state
+                 << " 'log_prob_state_given_other' "       << std::setw(wd) << log_prob_state_given_other
+                 << " '-log_importance_dens' "             << std::setw(wd) << -it->log_importance_dens;
+
+              if(did_resample)
+                ss << " 'parent->log_weight' "             << std::setw(wd) << it->parent->log_weight
+                   << " '-parent->log_resampling_weight' " << std::setw(wd) << -it->parent->log_resampling_weight;
+
+              if(!is_forward)
+                ss << " 'log_artificial_prior(t)' "        << std::setw(wd) <<  dens_calc.log_artificial_prior(*it        , t)
+                   << " '-log_artificial_prior(t + 1)' "   << std::setw(wd) << -dens_calc.log_artificial_prior(*it->parent, t + 1);
+
+              data.log(5) << ss.str();
+            };
 
           } // end loop over new particle
 
-#ifdef _OPENMP
-#pragma omp critical(aux_lock)
-{
-#endif
-          max_weight = MAX(my_max_weight, max_weight);
-#ifdef _OPENMP
-}
-} // end omp parallel
-#endif
-
           normalize_log_weights<false, true>(new_cloud, max_weight);
       }
 
@@ -179,9 +184,8 @@ public:
 */
 
 template<
-  template <typename, bool> class T_resampler,
-  template <typename, bool> class T_importance_dens,
-  class densities>
+  template <bool> class T_resampler,
+  template <bool> class T_importance_dens>
 class PF_smoother_Fearnhead_O_N : private PF_base {
   using uword = arma::uword;
 
@@ -198,16 +202,15 @@ class PF_smoother_Fearnhead_O_N : private PF_base {
   }
 
 public:
-  static smoother_output compute(const PF_data &data){
+  static smoother_output
+  compute(const PF_data &data, pf_base_dens &dens_calc){
     smoother_output result;
-    std::vector<cloud> &forward_clouds = result.forward_clouds;
+    std::vector<cloud> &forward_clouds  = result.forward_clouds;
     std::vector<cloud> &backward_clouds = result.backward_clouds;
     std::vector<cloud> &smoothed_clouds = result.smoothed_clouds;
 
-    forward_clouds  = forward_filter::compute(data);
-    backward_clouds = backward_filter::compute(data);
-
-    densities dens_calc = densities(data);
+    forward_clouds  =  forward_filter::compute(data, dens_calc);
+    backward_clouds = backward_filter::compute(data, dens_calc);
 
     if(data.debug > 0)
       data.log(1) << "Finished finding forward and backward clouds. Started smoothing";
@@ -248,57 +251,41 @@ public:
       if(data.debug > 0)
         data.log(1) << "Weighting particles";
       {
-          double max_weight = -std::numeric_limits<double>::max();
-          arma::uvec r_set = get_risk_set(data, t);
-          unsigned int n_elem = new_cloud.size();
+        double max_weight = -std::numeric_limits<double>::max();
+        arma::uvec r_set = get_risk_set(data, t);
+        unsigned int n_elem = new_cloud.size();
 
 #ifdef _OPENMP
-#pragma omp parallel
-{
+#pragma omp parallel for schedule(static) reduction(max:max_weight)
 #endif
-          double my_max_weight = -std::numeric_limits<double>::max();
+        for(unsigned int i = 0; i < n_elem; ++i){ // loop over smooth clouds
+          auto it = new_cloud.begin() + i;
+          double log_prob_y_given_state =
+            dens_calc.log_prob_y_given_state(
+              it->get_state(), t, r_set, false);
+          double log_prob_state_given_previous =
+            dens_calc.log_prob_state_given_previous(
+              it->get_state()       , it->parent->get_state(), t);
+          double log_prob_next_given_state =
+            dens_calc.log_prob_state_given_previous(
+              it->child->get_state(), it->get_state()        , t + 1);
+          double log_importance_dens = it->log_importance_dens;
+          double log_artificial_prior = // TODO: have already been computed
+            dens_calc.log_artificial_prior(
+              *it->child /* note child */, t + 1 /* note t + 1 */);
 
-#ifdef _OPENMP
-#pragma omp for schedule(static)
-#endif
-          for(unsigned int i = 0; i < n_elem; ++i){ // loop over smooth clouds
-            auto it = new_cloud.begin() + i;
-            double log_prob_y_given_state =
-              dens_calc.log_prob_y_given_state(
-                data, it->get_state(), t, r_set, false);
-            double log_prob_state_given_previous =
-              dens_calc.log_prob_state_given_previous(
-                data, it->get_state(), it->parent->get_state(), t);
-            double log_prob_next_given_state =
-              dens_calc.log_prob_state_given_next(
-                data, it->get_state(), it->child->get_state(), t);
-            double log_importance_dens = it->log_importance_dens;
-            double log_artificial_prior = // TODO: have already been computed
-              dens_calc.log_artificial_prior(
-                *it->child /* note child */, t + 1 /* note t + 1 */);
+          it->log_unnormalized_weight = it->log_weight =
+            /* nominator */
+            (log_prob_y_given_state + log_prob_state_given_previous + log_prob_next_given_state +
+              it->parent->log_weight + it->child->log_weight)
+            /* denoninator */
+            - (log_importance_dens + it->parent->log_resampling_weight +
+                it->child->log_resampling_weight + log_artificial_prior);
 
-            it->log_unnormalized_weight = it->log_weight =
-              /* nominator */
-              (log_prob_y_given_state + log_prob_state_given_previous + log_prob_next_given_state +
-                it->parent->log_weight + it->child->log_weight)
-              /* denoninator */
-              - (log_importance_dens + it->parent->log_resampling_weight +
-                  it->child->log_resampling_weight + log_artificial_prior);
+          max_weight = MAX(max_weight, it->log_weight);
+        } // end over smooth clouds
 
-            my_max_weight = MAX(it->log_weight, my_max_weight);
-          } // end over smooth clouds
-
-#ifdef _OPENMP
-#pragma omp critical(smoother_lock)
-{
-#endif
-          max_weight = MAX(max_weight, my_max_weight);
-#ifdef _OPENMP
-}
-} // end omp parallel
-#endif
-
-          normalize_log_weights<false, true>(new_cloud, max_weight);
+        normalize_log_weights<false, true>(new_cloud, max_weight);
       }
 
       debug_msg_after_weighting(data, new_cloud);
@@ -311,10 +298,10 @@ public:
   }
 
   using forward_filter =
-    AUX_PF<T_resampler, T_importance_dens, densities, true>;
+    AUX_PF<T_resampler, T_importance_dens, true>;
   using backward_filter =
-    AUX_PF<T_resampler, T_importance_dens, densities, false>;
-  using importance_dens = T_importance_dens<densities, false /* arg should not matter*/>;
+    AUX_PF<T_resampler, T_importance_dens, false>;
+  using importance_dens = T_importance_dens<false /* arg should not matter*/>;
 };
 
 /*
@@ -325,16 +312,16 @@ public:
 */
 
 template<
-  template <typename, bool> class T_resampler,
-  template <typename, bool> class T_importance_dens,
-  class densities>
+  template <bool> class T_resampler,
+  template <bool> class T_importance_dens>
 class PF_smoother_Brier_O_N_square : private PF_base {
   using uword = arma::uword;
 
 public:
-  static smoother_output compute(const PF_data &data){
+  static smoother_output
+  compute(const PF_data &data, pf_base_dens &dens_calc){
     smoother_output result;
-    std::vector<cloud> &forward_clouds = result.forward_clouds;
+    std::vector<cloud> &forward_clouds  = result.forward_clouds;
     std::vector<cloud> &backward_clouds = result.backward_clouds;
     std::vector<cloud> &smoothed_clouds = result.smoothed_clouds;
     std::shared_ptr<smoother_output::trans_like_obj> trans_ptr =
@@ -342,16 +329,16 @@ public:
 
     smoother_output::trans_like_obj &transition_likelihoods = *trans_ptr;
 
-    forward_clouds = forward_filter::compute(data);
-    backward_clouds = backward_filter::compute(data);
+    forward_clouds  = forward_filter::compute(data, dens_calc);
+    backward_clouds = backward_filter::compute(data, dens_calc);
 
     if(data.debug > 0)
       data.log(1) << "Finished finding forward and backward clouds. Started smoothing";
 
     auto fw_cloud = forward_clouds.begin();
-    //++fw_cloud; // first index is time 0 -- we need index 1 to start with
+    //++fw_cloud; // first index is time 0
     auto bw_cloud = backward_clouds.rbegin();
-    //++bw_cloud; // first index is time 1 -- we need index 2 to start with
+    //++bw_cloud; // first index is time 1
 
     double max_weight = -std::numeric_limits<double>::max();
     for(int t = 1; t <= data.d /* note the leq */; ++t, ++fw_cloud, ++bw_cloud){
@@ -391,13 +378,7 @@ public:
       cloud new_cloud(n_elem_bw);
 
 #ifdef _OPENMP
-#pragma omp parallel
-{
-#endif
-      densities dens_calc = densities(data);
-      double my_max_weight = -std::numeric_limits<double>::max();
-#ifdef _OPENMP
-#pragma omp for schedule(static)
+#pragma omp parallel for schedule(static) reduction(max:max_weight)
 #endif
       for(unsigned int i = 0; i < n_elem_bw; ++i){ // loop over bw particle
         particle &bw_particle = (*bw_cloud)[i];
@@ -411,7 +392,7 @@ public:
             ++fw_particle){
           // compute un-normalized weight of pair
           double log_like_transition = dens_calc.log_prob_state_given_previous(
-            data, bw_particle.get_state(), fw_particle->get_state(), t);
+            bw_particle.get_state(), fw_particle->get_state(), t);
           double this_term = fw_particle->log_weight + log_like_transition;
 
           // add pair information and update max log term seen so far
@@ -450,19 +431,9 @@ public:
         particle &new_p = new_cloud.set_particle(i, bw_particle.get_state());
         new_p.log_unnormalized_weight = new_p.log_weight = this_log_weight;
 
-        my_max_weight = MAX(my_max_weight, this_log_weight);
+        max_weight = MAX(max_weight, this_log_weight);
       } // end loop over bw particle
 
-#ifdef _OPENMP
-#pragma omp critical(smoother_lock_brier_three)
-{
-#endif
-      max_weight = MAX(max_weight, my_max_weight);
-#ifdef _OPENMP
-}
-} // end omp parallel
-#endif
-
       // normalize smoothed weights
       normalize_log_weights<false, true>(new_cloud, max_weight);
 
@@ -483,9 +454,9 @@ public:
   }
 
   using forward_filter =
-    AUX_PF<T_resampler, T_importance_dens, densities, true>;
+    AUX_PF<T_resampler, T_importance_dens, true>;
   using backward_filter =
-    AUX_PF<T_resampler, T_importance_dens, densities, false>;
+    AUX_PF<T_resampler, T_importance_dens, false>;
 };
 
 
